==> ascs-1.2.tar.gz: ascs-1.2/include/ascs/tcp/server.h <==
/*
 * server.h
 *
 *  Created on: 2012-3-2
 *      Author: youngwolf
 *		email: mail2tao@163.com
 *		QQ: 676218192
 *		Community on QQ: 198941541
 *
 * this class only used at server endpoint
 */

#ifndef _ASCS_SERVER_H_
#define _ASCS_SERVER_H_

#include "../object_pool.h"

namespace ascs { namespace tcp {

template<typename Socket, typename Pool = object_pool<Socket>, typename Server = i_server>
class server_base : public Server, public Pool
{
public:
	server_base(service_pump& service_pump_) : Pool(service_pump_), acceptor(service_pump_) {set_server_addr(ASCS_SERVER_PORT);}
	template<typename Arg>
	server_base(service_pump& service_pump_, const Arg& arg) : Pool(service_pump_, arg), acceptor(service_pump_) {set_server_addr(ASCS_SERVER_PORT);}

	bool set_server_addr(unsigned short port, const std::string& ip = std::string())
	{
		if (ip.empty())
			server_addr = asio::ip::tcp::endpoint(ASCS_TCP_DEFAULT_IP_VERSION, port);
		else
		{
			asio::error_code ec;
#if ASIO_VERSION >= 101100
			auto addr = asio::ip::make_address(ip, ec);
#else
			auto addr = asio::ip::address::from_string(ip, ec);
#endif
			if (ec)
				return false;

			server_addr = asio::ip::tcp::endpoint(addr, port);
		}

		return true;
	}
	const asio::ip::tcp::endpoint& get_server_addr() const {return server_addr;}

	void stop_listen() {asio::error_code ec; acceptor.cancel(ec); acceptor.close(ec);}
	bool is_listening() const {return acceptor.is_open();}

	//implement i_server's pure virtual functions
	virtual service_pump& get_service_pump() {return Pool::get_service_pump();}
	virtual const service_pump& get_service_pump() const {return Pool::get_service_pump();}
	virtual bool del_socket(const std::shared_ptr<object>& socket_ptr)
	{
		auto raw_socket_ptr(std::dynamic_pointer_cast<Socket>(socket_ptr));
		if (!raw_socket_ptr)
			return false;

		raw_socket_ptr->force_shutdown();
		return this->del_object(raw_socket_ptr);
	}
	//restore the invalid socket whose id is equal to id, if successful, socket_ptr's take_over function will be invoked,
	//you can restore the invalid socket to socket_ptr, everything is restorable except socket::next_layer_ (on the other
	//hand, restore socket::next_layer_ doesn't make any sense).
	virtual bool restore_socket(const std::shared_ptr<object>& socket_ptr, uint_fast64_t id)
	{
		auto raw_socket_ptr(std::dynamic_pointer_cast<Socket>(socket_ptr));
		if (!raw_socket_ptr)
			return false;

		auto this_id = raw_socket_ptr->id();
		auto old_socket_ptr = this->change_object_id(raw_socket_ptr, id);
		if (old_socket_ptr)
		{
			assert(raw_socket_ptr->id() == old_socket_ptr->id());

			unified_out::info_out("object id " ASCS_LLF " been reused, id " ASCS_LLF " been discarded.", raw_socket_ptr->id(), this_id);
			raw_socket_ptr->take_over(old_socket_ptr);

			return true;
		}

		return false;
	}

	///////////////////////////////////////////////////
	//msg sending interface
	TCP_BROADCAST_MSG(broadcast_msg, send_msg)
	TCP_BROADCAST_MSG(broadcast_native_msg, send_native_msg)
	//guarantee send msg successfully even if can_overflow equal to false
	//success at here just means putting the msg into tcp::socket_base's send buffer
	TCP_BROADCAST_MSG(safe_broadcast_msg, safe_send_msg)
	TCP_BROADCAST_MSG(safe_broadcast_native_msg, safe_send_native_msg)
	//send message with sync mode
	TCP_BROADCAST_MSG(sync_broadcast_msg, sync_send_msg)
	TCP_BROADCAST_MSG(sync_broadcast_native_msg, sync_send_native_msg)
	//msg sending interface
	///////////////////////////////////////////////////

	//functions with a socket_ptr parameter will remove the link from object pool first, then call corresponding function.
	void disconnect(typename Pool::object_ctype& socket_ptr) {this->del_object(socket_ptr); socket_ptr->disconnect();}
	void disconnect() {this->do_something_to_all([=](typename Pool::object_ctype& item) {item->disconnect();});}
	void force_shutdown(typename Pool::object_ctype& socket_ptr) {this->del_object(socket_ptr); socket_ptr->force_shutdown();}
	void force_shutdown() {this->do_something_to_all([=](typename Pool::object_ctype& item) {item->force_shutdown();});}
	void graceful_shutdown(typename Pool::object_ctype& socket_ptr, bool sync = false) {this->del_object(socket_ptr); socket_ptr->graceful_shutdown(sync);}
	void graceful_shutdown() {this->do_something_to_all([](typename Pool::object_ctype& item) {item->graceful_shutdown();});} //parameter sync must be false (the default value), or dead lock will occur.

protected:
	virtual bool init()
	{
		asio::error_code ec;
		if (!acceptor.is_open()) {acceptor.open(server_addr.protocol(), ec); assert(!ec);} //user maybe has opened this acceptor (to set options for example)
#ifndef ASCS_NOT_REUSE_ADDRESS
		acceptor.set_option(asio::ip::tcp::acceptor::reuse_address(true), ec); assert(!ec);
#endif
		acceptor.bind(server_addr, ec); assert(!ec);
		if (ec) {get_service_pump().stop(); unified_out::error_out("bind failed."); return false;}
#if ASIO_VERSION >= 101100
		acceptor.listen(asio::ip::tcp::acceptor::max_listen_connections, ec); assert(!ec);
#else
		acceptor.listen(asio::ip::tcp::acceptor::max_connections, ec); assert(!ec);
#endif
		if (ec) {get_service_pump().stop(); unified_out::error_out("listen failed."); return false;}

		this->start();

		for (auto i = 0; i < ASCS_ASYNC_ACCEPT_NUM; ++i)
			start_next_accept();

		return true;
	}
	virtual void uninit() {this->stop(); stop_listen(); force_shutdown();} //if you wanna graceful shutdown, call graceful_shutdown before service_pump::stop_service invocation.
	virtual bool on_accept(typename Pool::object_ctype& socket_ptr) {return true;}

	//if you want to ignore this error and continue to accept new connections immediately, return true in this virtual function;
	//if you want to ignore this error and continue to accept new connections after a specific delay, start a timer immediately and return false (don't call stop_listen()),
	// when the timer ends up, call start_next_accept() in the callback function.
	//otherwise, don't rewrite this virtual function or call server_base::on_accept_error() directly after your code.
	virtual bool on_accept_error(const asio::error_code& ec, typename Pool::object_ctype& socket_ptr)
	{
		if (asio::error::operation_aborted != ec)
		{
			unified_out::error_out("failed to accept new connection because of %s, will stop listening.", ec.message().data());
			stop_listen();
		}

		return false;
	}

	virtual void start_next_accept()
	{
		auto socket_ptr = this->create_object(*this);
		acceptor.async_accept(socket_ptr->lowest_layer(), [=](const asio::error_code& ec) {this->accept_handler(ec, socket_ptr);});
	}

protected:
	bool add_socket(typename Pool::object_ctype& socket_ptr)
	{
		if (this->add_object(socket_ptr))
		{
			socket_ptr->show_info("client:", "arrive.");
			return true;
		}

		socket_ptr->show_info("client:", "been refused because of too many clients.");
		socket_ptr->force_shutdown();
		return false;
	}

	void accept_handler(const asio::error_code& ec, typename Pool::object_ctype& socket_ptr)
	{
		if (!ec)
		{
			if (on_accept(socket_ptr) && add_socket(socket_ptr))
				socket_ptr->start();

			start_next_accept();
		}
		else if (on_accept_error(ec, socket_ptr))
			start_next_accept();
	}

protected:
	asio::ip::tcp::endpoint server_addr;
	asio::ip::tcp::acceptor acceptor;
};

}} //namespace

#endif /* _ASCS_SERVER_H_ */

==> ascs-1.3.tar.gz: ascs-1.3/include/ascs/tcp/server.h <==
/*
 * server.h
 *
 *  Created on: 2012-3-2
 *      Author: youngwolf
 *		email: mail2tao@163.com
 *		QQ: 676218192
 *		Community on QQ: 198941541
 *
 * this class only used at server endpoint
 */

#ifndef _ASCS_SERVER_H_
#define _ASCS_SERVER_H_

#include "../object_pool.h"

namespace ascs { namespace tcp {

template<typename Socket, typename Pool = object_pool<Socket>, typename Server = i_server>
class server_base : public Server, public Pool
{
public:
	server_base(service_pump& service_pump_) : Pool(service_pump_), acceptor(service_pump_) {set_server_addr(ASCS_SERVER_PORT);}
	template<typename Arg>
	server_base(service_pump& service_pump_, const Arg& arg) : Pool(service_pump_, arg), acceptor(service_pump_) {set_server_addr(ASCS_SERVER_PORT);}

	bool set_server_addr(unsigned short port, const std::string& ip = std::string())
	{
		if (ip.empty())
			server_addr = asio::ip::tcp::endpoint(ASCS_TCP_DEFAULT_IP_VERSION, port);
		else
		{
			asio::error_code ec;
#if ASIO_VERSION >= 101100
			auto addr = asio::ip::make_address(ip, ec);
#else
			auto addr = asio::ip::address::from_string(ip, ec);
#endif
			if (ec)
				return false;

			server_addr = asio::ip::tcp::endpoint(addr, port);
		}

		return true;
	}
	const asio::ip::tcp::endpoint& get_server_addr() const {return server_addr;}

	void stop_listen() {asio::error_code ec; acceptor.cancel(ec); acceptor.close(ec);}
	bool is_listening() const {return acceptor.is_open();}

	asio::ip::tcp::acceptor& next_layer() {return acceptor;}
	const asio::ip::tcp::acceptor& next_layer() const {return acceptor;}

	//implement i_server's pure virtual functions
	virtual service_pump& get_service_pump() {return Pool::get_service_pump();}
	virtual const service_pump& get_service_pump() const {return Pool::get_service_pump();}
	virtual bool del_socket(const std::shared_ptr<tracked_executor>& socket_ptr)
	{
		auto raw_socket_ptr(std::dynamic_pointer_cast<Socket>(socket_ptr));
		if (!raw_socket_ptr)
			return false;

		raw_socket_ptr->force_shutdown();
		return this->del_object(raw_socket_ptr);
	}
	//restore the invalid socket whose id is equal to id, if successful, socket_ptr's take_over function will be invoked,
	//you can restore the invalid socket to socket_ptr, everything can be restored except socket::next_layer_ (on the other
	//hand, restore socket::next_layer_ doesn't make any sense).
	virtual bool restore_socket(const std::shared_ptr<tracked_executor>& socket_ptr, uint_fast64_t id)
	{
		auto raw_socket_ptr(std::dynamic_pointer_cast<Socket>(socket_ptr));
		if (!raw_socket_ptr)
			return false;

		auto this_id = raw_socket_ptr->id();
		auto old_socket_ptr = this->change_object_id(raw_socket_ptr, id);
		if (old_socket_ptr)
		{
			assert(raw_socket_ptr->id() == old_socket_ptr->id());

			unified_out::info_out("object id " ASCS_LLF " been reused, id " ASCS_LLF " been discarded.", raw_socket_ptr->id(), this_id);
			raw_socket_ptr->take_over(old_socket_ptr);

			return true;
		}

		return false;
	}
	virtual std::shared_ptr<tracked_executor> find_socket(uint_fast64_t id) {return this->find(id);}

	///////////////////////////////////////////////////
	//msg sending interface
	TCP_BROADCAST_MSG(broadcast_msg, send_msg)
	TCP_BROADCAST_MSG(broadcast_native_msg, send_native_msg)
	//guarantee send msg successfully even if can_overflow equal to false
	//success at here just means putting the msg into tcp::socket_base's send buffer
	TCP_BROADCAST_MSG(safe_broadcast_msg, safe_send_msg)
	TCP_BROADCAST_MSG(safe_broadcast_native_msg, safe_send_native_msg)
	//msg sending interface
	///////////////////////////////////////////////////

	//functions with a socket_ptr parameter will remove the link from object pool first, then call corresponding function.
	void disconnect(typename Pool::object_ctype& socket_ptr) {this->del_object(socket_ptr); socket_ptr->disconnect();}
	void disconnect() {this->do_something_to_all([=](typename Pool::object_ctype& item) {item->disconnect();});}
	void force_shutdown(typename Pool::object_ctype& socket_ptr) {this->del_object(socket_ptr); socket_ptr->force_shutdown();}
	void force_shutdown() {this->do_something_to_all([=](typename Pool::object_ctype& item) {item->force_shutdown();});}
	void graceful_shutdown(typename Pool::object_ctype& socket_ptr, bool sync = false) {this->del_object(socket_ptr); socket_ptr->graceful_shutdown(sync);}
	void graceful_shutdown() {this->do_something_to_all([](typename Pool::object_ctype& item) {item->graceful_shutdown();});} //parameter sync must be false (the default value), or dead lock will occur.

protected:
	virtual int async_accept_num() {return ASCS_ASYNC_ACCEPT_NUM;}
	virtual bool init()
	{
		asio::error_code ec;
		if (!acceptor.is_open()) {acceptor.open(server_addr.protocol(), ec); assert(!ec);} //user maybe has opened this acceptor (to set options for example)
#ifndef ASCS_NOT_REUSE_ADDRESS
		acceptor.set_option(asio::ip::tcp::acceptor::reuse_address(true), ec); assert(!ec);
#endif
		acceptor.bind(server_addr, ec); assert(!ec);
		if (ec) {unified_out::error_out("bind failed."); return false;}

		auto num = async_accept_num();
		assert(num > 0);
		if (num <= 0)
			num = 16;

		std::list<typename Pool::object_type> sockets;
		unified_out::info_out("begin to pre-create %d server socket...", num);
		while (--num >= 0)
		{
			auto socket_ptr(this->create_object(*this));
			if (!socket_ptr)
				break;

			sockets.push_back(std::move(socket_ptr));
		}
		if (num >= 0)
			unified_out::info_out("finished pre-creating server sockets, but failed %d time(s).", num + 1);
		else
			unified_out::info_out("finished pre-creating server sockets.");

#if ASIO_VERSION >= 101100
		acceptor.listen(asio::ip::tcp::acceptor::max_listen_connections, ec); assert(!ec);
#else
		acceptor.listen(asio::ip::tcp::acceptor::max_connections, ec); assert(!ec);
#endif
		if (ec) {unified_out::error_out("listen failed."); return false;}

		ascs::do_something_to_all(sockets, [this](typename Pool::object_ctype& item) {this->do_async_accept(item);});
		this->start();

		return true;
	}
	virtual void uninit() {this->stop(); stop_listen(); force_shutdown();} //if you wanna graceful shutdown, call graceful_shutdown before service_pump::stop_service invocation.

	virtual bool on_accept(typename Pool::object_ctype& socket_ptr) {return true;}
	virtual void start_next_accept() {do_async_accept(this->create_object(*this));}

	//if you want to ignore this error and continue to accept new connections immediately, return true in this virtual function;
	//if you want to ignore this error and continue to accept new connections after a specific delay, start a timer immediately and return false (don't call stop_listen()),
	// after the timer exhausts, call start_next_accept() in the callback function.
	//otherwise, don't rewrite this virtual function or call server_base::on_accept_error() directly after your code.
	virtual bool on_accept_error(const asio::error_code& ec, typename Pool::object_ctype& socket_ptr)
	{
		if (asio::error::operation_aborted != ec)
		{
			unified_out::error_out("failed to accept new connection because of %s, will stop listening.", ec.message().data());
			stop_listen();
		}

		return false;
	}

protected:
	bool add_socket(typename Pool::object_ctype& socket_ptr)
	{
		if (this->add_object(socket_ptr))
		{
			socket_ptr->show_info("client:", "arrive.");
			return true;
		}

		socket_ptr->show_info("client:", "been refused because of too many clients.");
		socket_ptr->force_shutdown();
		return false;
	}

	void accept_handler(const asio::error_code& ec, typename Pool::object_ctype& socket_ptr)
	{
		if (!ec)
		{
			if (on_accept(socket_ptr) && add_socket(socket_ptr))
				socket_ptr->start();

			start_next_accept();
		}
		else if (on_accept_error(ec, socket_ptr))
			start_next_accept();
	}

private:
	void do_async_accept(typename Pool::object_ctype& socket_ptr)
		{if (socket_ptr) acceptor.async_accept(socket_ptr->lowest_layer(), [=](const asio::error_code& ec) {this->accept_handler(ec, socket_ptr);});}

private:
	asio::ip::tcp::endpoint server_addr;
	asio::ip::tcp::acceptor acceptor;
};

}} //namespace

#endif /* _ASCS_SERVER_H_ */
